/**
 *
 * Project: Eight Track, File: DiscordEventHandler.java
 *
 * Copyright 2019-2019 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.handlers;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import net.dv8tion.jda.api.entities.ChannelType;
import net.dv8tion.jda.api.entities.TextChannel;
import net.dv8tion.jda.api.events.guild.GuildJoinEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.events.message.react.MessageReactionAddEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.locusworks.discord.eighttrack.audio.Mp3UploadHandler;
import net.locusworks.discord.eighttrack.database.entities.DiscordGuild;
import net.locusworks.discord.eighttrack.services.ConfigurationService;
import net.locusworks.discord.eighttrack.services.GuildMusicService;
import net.locusworks.discord.eighttrack.services.GuildSongRepoService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Service
public class DiscordEventHandler extends ListenerAdapter {

  private Path musicDir;

  private ApplicationLogger logger = ApplicationLoggerFactory.getLogger(DiscordEventHandler.class);
  
  @Autowired 
  private ConfigurationService confService;

  @Autowired
  private Mp3UploadHandler uploadHandler;
  
  @Autowired
  private GuildSongRepoService guildSongRepoService;
  
  @Autowired
  private ReactionHandler reactionHandler;
  
  @PostConstruct
  private void init() throws IOException {
    this.musicDir = confService.getMusicDirectory();
    if (!Files.exists(this.musicDir.toAbsolutePath())) {
      Files.createDirectories(this.musicDir.toAbsolutePath());
    }
  }
  
  @Scheduled(fixedRate = 300000L, initialDelay=300000L)
  private void clearHandlerCache() {
    logger.debug("Clearing reaction handler cache of dead reactions");
    reactionHandler.cleanCache();
  }
  
  @Override
  public void onMessageReactionAdd(MessageReactionAddEvent e) {
    if (e.getUser().isBot()) {
      return;
    }
    if (!e.getChannel().getType().equals(ChannelType.TEXT)) {
      return;
    }
    TextChannel channel = (TextChannel) e.getChannel();
    if (reactionHandler.canHandle(channel.getGuild().getIdLong(), e.getMessageIdLong())) {
      reactionHandler.handle(channel, e.getMessageIdLong(), e.getUser().getIdLong(), e.getReaction());
    }
  }
  
  @Override
  public void onGuildJoin(GuildJoinEvent event) {
    try {
      long guildId = event.getGuild().getIdLong();
      DiscordGuild discordGuild =  guildSongRepoService.getGuildRepo().findByGuildId(guildId);
      if (discordGuild != null) return;

      logger.debug("Joining Server: " + event.getGuild().getName());

      discordGuild = new DiscordGuild();
      discordGuild.setGuildId(guildId);
      discordGuild.setGuildName(event.getGuild().getName());
      discordGuild.setDateJoined(new Date());

      guildSongRepoService.getGuildRepo().save(discordGuild);
    } catch (Exception ex) {
      logger.error("Unable to persist server information to database: " + ex.getMessage(), ex);
    }
  }

  @Override
  public void onGuildMessageReceived(GuildMessageReceivedEvent event) {
    if (event.getAuthor().isBot()) return;
    try {
      onGuildMessageReceivedHelper(event);
    } catch (Throwable t) {
      logger.error(t);
    }
  }

  private void onGuildMessageReceivedHelper(GuildMessageReceivedEvent event) throws Exception {
    
    if (!GuildMusicService.getMap().containsKey(event.getGuild().getIdLong())) {
      GuildMusicService.getMap().put(event.getGuild().getIdLong(), 
          new GuildMusicHandler(musicDir, event.getGuild().getIdLong(), uploadHandler, reactionHandler, guildSongRepoService));
    }
    
    GuildMusicHandler gmh = GuildMusicService.getMap().get(event.getGuild().getIdLong());
    gmh.accept((id) -> GuildMusicService.getMap().remove(id));
    
    
    List<String> commands = new ArrayList<String>(Arrays.asList(event.getMessage().getContentRaw().trim().toLowerCase().split(" ")));
    
    String command = commands.remove(0);
    
    switch(command) {
      case "+upload":
        gmh.upload(event);
        return;
      case "-play":
        gmh.play(event, commands);
        gmh.isPlaying(true);
        return;
      case "-stop":
        gmh.isPlaying(false);
        gmh.stop(event);
        event.getGuild().getAudioManager().closeAudioConnection();
        return;
      case "-next":
        gmh.next(event, commands);
        return;
      case "-repeat":
        gmh.repeat(event);
        gmh.isPlaying(true);
        return;
      case "-upnext":
        gmh.upNext(event, commands);
        return;
      case "-list":
        gmh.list(event);
        return;
      case "-playlist":
        gmh.playlist(event, commands);
        return;
      case "+playlist":
        gmh.adminPlaylist(event, commands);
        return;
      case "+delete":
        gmh.delete(event, commands);
        return;
      default:
        return;
    }
  }
}
